#include <Wire.h> 
#include <LCD_I2C.h>
#include <HX711_ADC.h>
LCD_I2C lcd(0x27,20,4);  // set the LCD address to 0x27 for a 16 chars and 2 line display
#define TRIG_PIN 5 // ESP32 pin GIOP23 connected to Ultrasonic Sensor's TRIG pin
#define ECHO_PIN 18 // ESP32 pin GIOP22 connected to Ultrasonic Sensor's ECHO pin
unsigned long t = 0;
 int i = 0;
float duration_us, distance_cm;
const int HX711_dout = 16; //mcu > HX711 dout pin
const int HX711_sck = 17; //mcu > HX711 sck pin
long current_time=0;
long interval=5000;
String Group="";
int screen_stage=0;
float height;
float weight;
float BMI;
HX711_ADC LoadCell(HX711_dout, HX711_sck);

float calibrationValue= 22.27;

void setup() {
  // put your setup code here, to run once:
    Serial.begin(115200);
  lcd.begin();
  lcd.backlight();
  pinMode(TRIG_PIN, OUTPUT);
  pinMode(ECHO_PIN, INPUT);
  boot_screen();
  
  LoadCell.begin();
  //LoadCell.setReverseOutput();
  unsigned long stabilizingtime = 2000; // tare preciscion can be improved by adding a few seconds of stabilizing time
  boolean _tare = true; //set this to false if you don't want tare to be performed in the next step
  LoadCell.start(stabilizingtime, _tare);
  if (LoadCell.getTareTimeoutFlag()) {
    Serial.println("Timeout, check MCU>HX711 wiring and pin designations");
  }
  else {
    LoadCell.setCalFactor(calibrationValue); // set calibration factor (float)
    Serial.println("Startup is complete");
  }
  while (!LoadCell.update());
  Serial.print("Calibration value: ");
  Serial.println(LoadCell.getCalFactor());
  Serial.print("HX711 measured conversion time ms: ");
  Serial.println(LoadCell.getConversionTime());
  Serial.print("HX711 measured sampling rate HZ: ");
  Serial.println(LoadCell.getSPS());
  Serial.print("HX711 measured settlingtime ms: ");
  Serial.println(LoadCell.getSettlingTime());
  Serial.println("Note that the settling time may increase significantly if you use delay() in your sketch!");
  if (LoadCell.getSPS() < 7) {
    Serial.println("!!Sampling rate is lower than specification, check MCU>HX711 wiring and pin designations");
  }
  else if (LoadCell.getSPS() > 100) {
    Serial.println("!!Sampling rate is higher than specification, check MCU>HX711 wiring and pin designations");
  }
    lcd.clear();

machine_ready();

}

void loop() {
  current_time=millis();
 
  /////////Check weight first///////
  while(screen_stage==0){
    if(millis()-current_time<=interval){
      check_weight();
      display_weight();
    }
    else{
      current_time=millis();
      screen_stage=1;
      lcd.clear();
    }
  }
  


 /////////Check Height Second///////
  while(screen_stage==1){
    if(millis()-current_time<=interval){
      check_height();
      display_height();
    }
    else{
      current_time=millis();
      screen_stage=2;
      lcd.clear();
    }
  }

   /////////Check BMI last///////
  while(screen_stage==2){
    if(millis()-current_time<=interval){
      display_waiting();
      check_BMI();
    }
    else{
      current_time=millis();
      screen_stage=3;
      BMI_screen();
    }
  }
}



void BMI_screen(){
  lcd.clear();
  lcd.setCursor(0,0);
  lcd.print("Height="+String(height)+"M");
  lcd.setCursor(0,1);
  lcd.print("Weight="+String(weight)+"Kg");
  lcd.setCursor(0,2);
  lcd.print("BMI="+String(BMI));
  lcd.setCursor(0,3);
  lcd.print("Group="+String(Group));
}

 
void boot_screen(){
  lcd.clear();
  lcd.setCursor(3,1);
  lcd.print("Welcome to");
  lcd.setCursor(2,2);
  lcd.print("BMI Machine");
  delay(3000);
}
void machine_ready(){
  lcd.clear();
  lcd.setCursor(5,1);
  lcd.print("Stand on");
  lcd.setCursor(2,2);
  lcd.print("BMI Machine");
  delay(10000);
}

void display_weight(){
   
  lcd.setCursor(3,0);
  lcd.print("Checking Weight");
  lcd.setCursor(3,2);
  lcd.print("Weight="+String(weight)+"Kg");
}

void display_height(){
  
  lcd.setCursor(3,0);
  lcd.print("Checking Height");
  lcd.setCursor(0,2);
 lcd.print("Height="+String(height)+"M");
}

void display_waiting(){
  
  lcd.setCursor(3,1);
  lcd.print("Wait for your");
  lcd.setCursor(3,2);
  lcd.print("Result...");
}


//////////////////////////////////////////////////////////////////////////////////////
    ////////////////////         CHECKING HEIGHT       ///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////

void check_height(){
   digitalWrite(TRIG_PIN, HIGH);
  delayMicroseconds(10);
  digitalWrite(TRIG_PIN, LOW);
  duration_us = pulseIn(ECHO_PIN, HIGH);
  distance_cm = 0.017 * duration_us;
  float dis=distance_cm/100;
  height = 2.03-(dis);
  
}

//////////////////////////////////////////////////////////////////////////////////////
    ////////////////////         CHECKING WEIGHT       ///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
void check_weight(){
    static boolean newDataReady = 0;
  const int serialPrintInterval = 500; //increase value to slow down serial print activity

  // check for new data/start next conversion:
  if (LoadCell.update()) newDataReady = true;

  // get smoothed value from the dataset:
  if (newDataReady) {
    if (millis() > t + serialPrintInterval) {
      weight = (LoadCell.getData())/1000;
      if(weight<=5){weight=0;}
      newDataReady = 0;
      t = millis();
    }
  }

  // receive command from serial terminal, send 't' to initiate tare operation:
  if (Serial.available() > 0) {
    char inByte = Serial.read();
    if (inByte == 't') LoadCell.tareNoDelay();
  }

  // check if last tare operation is complete:
  if (LoadCell.getTareStatus() == true) {
    Serial.println("Tare complete");
  }

}

//////////////////////////////////////////////////////////////////////////////////////
    ////////////////////         CHECKING BMI       ///////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////
void check_BMI(){
      BMI= BMI = (weight)/(height*height);
      if(BMI<18.5){
        Group="Under Weight";
      }else if(BMI>=18.5 && BMI<=24.9){
        Group="Healthy";
      }else if(BMI>25 && BMI<=29.9){
        Group="Over weight";
      }
      else{
        Group="Obesity";
      }
}
